@@ -7,7 +7,7 @@
 
 open Printf;
 
-value version = "8.02.01";
+value version = "8.03.00";
 value syntax_name = ref "";
 
 value ocaml_version =
@@ -26,7 +26,7 @@ value gram =
     {Plexing.tok_func _ = failwith "no loaded parsing module";
      Plexing.tok_using _ = (); Plexing.tok_removing _ = ();
      Plexing.tok_match = fun []; Plexing.tok_text _ = "";
-     Plexing.tok_comm = None}
+     Plexing.tok_comm = None; Plexing.kwds = Hashtbl.create 23 }
 ;
 
 (*
