@@ -1371,6 +1371,8 @@ EXTEND
     [ LEFTA
       [ me1 = extended_longident ; "." ; i = V LIDENT "lid" → 
           <:ctyp< $longid:me1$ . $_lid:i$ >>
+      | me1 = extended_longident ; "." ; t = paren_ctyp → 
+        <:ctyp< $longid:me1$ . ( $t$ ) >>
       | i = V LIDENT "lid" → 
           <:ctyp< $_lid:i$ >>
       ] 
@@ -1404,10 +1406,7 @@ EXTEND
       | ".." -> <:ctyp< .. >>
       | "_" → <:ctyp< _ >>
       | e = alg_extension -> <:ctyp< [% $_extension:e$ ] >>
-      | "(" ; "module"; mt = module_type ; ")" → <:ctyp< ( module $mt$ ) >>
-      | "("; t = SELF; "*"; tl = LIST1 ctyp SEP "*"; ")" → mktuptyp loc t tl
-      | "("; t = SELF; ")" → <:ctyp< $t$ >>
-      | "("; tl = V (LIST1 ctyp SEP "*"); ")" → <:ctyp< ( $_list:tl$ ) >>
+      | t = paren_ctyp -> t
       | "["; cdl = V (LIST1 constructor_declaration SEP "|"); "]" →
           <:ctyp< [ $_list:cdl$ ] >>
       | "["; "|"; "]" →
@@ -1415,6 +1414,13 @@ EXTEND
       | "{"; ldl = V (LIST1 label_declaration SEP ";"); "}" →
           <:ctyp< { $_list:ldl$ } >> ] ]
   ;
+  paren_ctyp:
+    [ [ "(" ; "module"; mt = module_type ; ")" → <:ctyp< ( module $mt$ ) >>
+      | "("; t = ctyp; "*"; tl = LIST1 ctyp SEP "*"; ")" → mktuptyp loc t tl
+      | "("; t = ctyp; ")" → <:ctyp< $t$ >>
+      | "("; tl = V (LIST1 ctyp SEP "*"); ")" → <:ctyp< ( $_list:tl$ ) >>
+    ] ]
+  ;
   ctyp_below_alg_attribute:
   [ [ x = ctyp LEVEL "below_alg_attribute" -> x ]
   ]
